@@ -1,15 +1,18 @@
 package Module::Pluggable;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION $FORCE_SEARCH_ALL_PATHS);
 use Module::Pluggable::Object;
 
+use if $] > 5.017, 'deprecate';
+
 # ObQuote:
 # Bob Porter: Looks like you've been missing a lot of work lately.
 # Peter Gibbons: I wouldn't say I've been missing it, Bob!
 
 
-$VERSION = '3.9';
+$VERSION = '5.2';
+$FORCE_SEARCH_ALL_PATHS = 0;
 
 sub import {
     my $class        = shift;
@@ -22,6 +25,7 @@ sub import {
     my ($package)    = $opts{'package'} || $pkg;
     $opts{filename}  = $file;
     $opts{package}   = $package;
+    $opts{force_search_all_paths} = $FORCE_SEARCH_ALL_PATHS unless exists $opts{force_search_all_paths};
 
 
     my $finder       = Module::Pluggable::Object->new(%opts);
@@ -124,10 +128,10 @@ core code. For that, you might do something like ...
 
 .. and all the plugins will get a chance in turn to look at it.
 
-This can be trivally extended so that plugins could save the email
+This can be trivially extended so that plugins could save the email
 somewhere and then no other plugin should try and do that.
 Simply have it so that the C<examine> method returns C<1> if
-it has saved the email somewhere. You might also wnat to be paranoid
+it has saved the email somewhere. You might also want to be paranoid
 and check to see if the plugin has an C<examine> method.
 
         foreach my $plugin ($self->plugins) {
@@ -152,7 +156,6 @@ Optionally it instantiates those classes for you.
 
 =head1 ADVANCED USAGE
 
-
 Alternatively, if you don't want to use 'plugins' as the method ...
 
     package MyClass;
@@ -228,6 +231,21 @@ and then later ...
     my @filters = $self->filters;
     my @plugins = $self->plugins;
 
+=head1 PLUGIN SEARCHING
+
+Every time you call 'plugins' the whole search path is walked again. This allows
+for dynamically loading plugins even at run time. However this can get expensive
+and so if you don't expect to want to add new plugins at run time you could do
+
+
+  package Foo;
+  use strict;
+  use Module::Pluggable sub_name => '_plugins';
+
+  our @PLUGINS;
+  sub plugins { @PLUGINS ||= shift->_plugins }
+  1;
+
 =head1 INNER PACKAGES
 
 If you have, for example, a file B<lib/Something/Plugin/Foo.pm> that
@@ -254,7 +272,7 @@ An array ref of namespaces to look in.
 
 =head2 search_dirs
 
-An array ref of directorys to look in before @INC.
+An array ref of directories to look in before @INC.
 
 =head2 instantiate
 
@@ -307,6 +325,62 @@ the extensions F<.swp> or F<.swo>, or files beginning with F<.#>.
 Setting C<include_editor_junk> changes C<Module::Pluggable> so it does
 not ignore any files it finds.
 
+=head2 follow_symlinks
+
+Whether, when searching directories, to follow symlinks.
+
+Defaults to 1 i.e do follow symlinks.
+
+=head2 min_depth, max_depth
+
+This will allow you to set what 'depth' of plugin will be allowed.
+
+So, for example, C<MyClass::Plugin::Foo> will have a depth of 3 and
+C<MyClass::Plugin::Foo::Bar> will have a depth of 4 so to only get the former
+(i.e C<MyClass::Plugin::Foo>) do
+
+        package MyClass;
+        use Module::Pluggable max_depth => 3;
+
+and to only get the latter (i.e C<MyClass::Plugin::Foo::Bar>)
+
+        package MyClass;
+        use Module::Pluggable min_depth => 4;
+
+
+=head1 TRIGGERS
+
+Various triggers can also be passed in to the options.
+
+If any of these triggers return 0 then the plugin will not be returned.
+
+=head2 before_require <plugin>
+
+Gets passed the plugin name.
+
+If 0 is returned then this plugin will not be required either.
+
+=head2 on_require_error <plugin> <err>
+
+Gets called when there's an error on requiring the plugin.
+
+Gets passed the plugin name and the error.
+
+The default on_require_error handler is to C<carp> the error and return 0.
+
+=head2 on_instantiate_error <plugin> <err>
+
+Gets called when there's an error on instantiating the plugin.
+
+Gets passed the plugin name and the error.
+
+The default on_instantiate_error handler is to C<carp> the error and return 0.
+
+=head2 after_require <plugin>
+
+Gets passed the plugin name.
+
+If 0 is returned then this plugin will be required but not returned as a plugin.
 
 =head1 METHODs
 
@@ -319,18 +393,60 @@ search_path.
     $self->search_path( add => "New::Path" ); # add
     $self->search_path( new => "New::Path" ); # replace
 
+=head1 BEHAVIOUR UNDER TEST ENVIRONMENT
 
+In order to make testing reliable we exclude anything not from blib if blib.pm is
+in %INC.
+
+However if the module being tested used another module that itself used C<Module::Pluggable>
+then the second module would fail. This was fixed by checking to see if the caller
+had (^|/)blib/ in their filename.
+
+There's an argument that this is the wrong behaviour and that modules should explicitly
+trigger this behaviour but that particular code has been around for 7 years now and I'm
+reluctant to change the default behaviour.
+
+You can now (as of version 4.1) force Module::Pluggable to look outside blib in a test environment by doing either
+
+        require Module::Pluggable;
+        $Module::Pluggable::FORCE_SEARCH_ALL_PATHS = 1;
+        import Module::Pluggable;
+
+or
+
+        use Module::Pluggable force_search_all_paths => 1;
+
+=head1 @INC hooks and App::FatPacker
+
+If a module's @INC has a hook and that hook is an object which has a C<files()> method then we will
+try and require those files too. See C<t/26inc_hook.t> for an example.
+
+This has allowed L<App::FatPacker> (as of version 0.10.0) to provide support for Module::Pluggable.
+
+This should also, theoretically, allow someone to modify PAR to do the same thing.
+
+=head1 Module::Require recommended
+
+Up until version 5.2 L<Module::Pluggable> used a string C<eval> to require plugins.
+
+This has now been changed to optionally use L<Module::Runtime> and it's C<require_module> method when
+available and fall back to using a path based C<require> when not.
+
+It's recommended, but not required, that you install Module::Runtime.
 
 =head1 FUTURE PLANS
 
 This does everything I need and I can't really think of any other
-features I want to add. Famous last words of course
+features I want to add. Famous last words of course (not least
+because we're up to version 5.0 at the time of writing).
 
-Recently tried fixed to find inner packages and to make it
-'just work' with PAR but there are still some issues.
+However suggestions (and patches) are always welcome.
 
+=head1 DEVELOPMENT
 
-However suggestions (and patches) are welcome.
+The master repo for this module is at
+
+https://github.com/simonwistow/Module-Pluggable
 
 =head1 AUTHOR
 
